;---------------------------------------------------------------------------;
;                                                                           ;
;   SQ.LSP   Version 1.1                                                    ;
;                                                                           ;
;     C.J.Latham, 15A Clifton Villas, London W9 2PH Tel: 0171 289 0402      ;
;                                                                           ;
;     I would like to hear from you if you find this "sq.lsp" useful        ;                                                                      
;     criticisms & suggestions equally are welcomed. Upgrades may           ;
;     become available in due course.                                       ;
;                                                                           ;
;     Permission to use, copy, modify, and distribute this software         ;
;     for any purpose and without fee is hereby granted, provided that      ;
;     the above name & address appears in all copies and that revisions     ;
;     are named, dated & noted. Both the name, address and this             ;
;     permission notice should appear in all supporting documentation.      ;                                     
;                                                                           ;
;     THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED          ;
;     WARRANTY.  ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR       ;
;     PURPOSE ARE HEREBY DISCLAIMED.                                        ;
;                                                                           ;
;   Created by: Chris Latham   Date: 2 Oct 1995                             ;                              
;   Revised by: Chris Latham   Date: 5 Oct 1995                             ;      
;                                                                           ;
;---------------------------------------------------------------------------;
;                                                                           ;
;  DESCRIPTION                                                              ;
;                                                                           ;
;   Inserts a sequence number block into a drawing; user asked if a new     ;                     
;   size is required and the change can be saved for future insertions.     ;
;   Increments sequence numbers starting at numbers provided by files with  ;
;   extension "*.seq" which can be created to hold a series of sequence     ;
;   numbers relevant to a particular project and saved in any directory.    ;
;   The current sequence file (called "curr.seq") should be in the root     ;
;   directory, "c:\", "curr.seq" to be read by this function (so far)       ;
;   and can be changed to contain sequences of an alternative project or    ;
;   a new project sequence file created and loaded into "curr.seq".         ;
;   Sequences can be restarted at a new number, removed or added to the     ;
;   file. The first line of all "*.seq" files hold the filename and path    ;
;   where the parent file should reside, so the "curr.seq" first line shows ;  
;   which file is current and where it's parent is to be found. The second  ;
;   line is for the default user key. When the current sequence file is     ;
;   altered, the alterations are copied back to the parent file.            ;
;   A WBLOCK (as a drawing called "SQ.dwg") should be installed that is     ;      
;   used to hold the sequence number and prefix in a rounded box.           ;
;                                                                           ;
;     Improvements are anticipated; Eg:  1. Coding can be tidier &          ;                                                            
;     shorter: a) by using more shared code in sub-functions; b) using      ;        
;     the "initget" function should simplify user input functions c) "list" ;
;     functions may simplify some aspects  2. The function could be         ;
;     included in a menu  3. It may, in future, ask if the user wants a     ;                                         
;     leader to the object being labelled with a sequence block.            ;
;     4. A future version may do without the WBLOCK altogether and create   ;
;     a text box with arcs & lines just outside the text, this will have    ;
;     several advantages or it might be an anonymous block.  5. A dialog    ;             
;     entry version could emerge.  6. The prompt string (and the file)      ;                                          
;     for the command should be in alphabetical order.                       ;
;                                                                           ;
;---------------------------------------------------------------------------;
;  Error function handler substituted for autoCAD internal error handling
;  that resets some system variables before running rest of "c:sq"
;
(defun sqerr (s)
   ; if other than "Function Cancelled" 
   (if (/= s "Function cancelled")  
      (princ (strcat "\nError: " s)) ; print the error to screen
   ) ; end if
   (setq *error* olderr) ; sets error handling to previous
   (echblpres) ;reset "blip" & "echo" modes to previous  
   (princ)
) ; end sqerr
;--------------------------------------------------------------------------;
;  Redirects errors to function above; turns off echo & blip modes;
;  builds prompt string from current sequence file, directs response
;  by user to functions appropriate to the user's choices
;
(defun c:sq ()
   ; save error setting to variable
   (setq olderr *error* *error*  sqerr) ; and redirect errors to subroutine 
   ;the name of the current sequence file to which any requested
   ;sequence file is copied for current use
   (setq filename "C:\CURR.SEQ") ; (could be changed to a different path) 
   (setq fil (findfile filename)) ;set to fully qualified pathname
   (if fil ; if the file was found
     (setq filename fil)
     ; else
     (nofilesq) ; otherwise do the "Cannot find file" function
   ) ; end if
   (echblpoff) ;function to chech current "blipmode" & "echomode"
   ; and store these values in global variables to return to
   ; and turn "blipmode" & "echomode" off until finished
   (setq fr (open filename "r"))   
   (if fr ; if file opened sucessfully
     (progn  
       (setq sqpf (read-line fr)) ;read first line  - path & file name
       ; strips path from "sqpf" for use in prompt line
       (setq sqf (strend sqpf "\\")) ;a "\" string written as "\\" in autoLISP 
       (setq sqd (read-line fr)) ;read second line - default key 
       (setq prmpt "/") ;initialise variable on which to build prompt line
       (while (setq ln (read-line fr))
         (setq lprmpt (strcol ln 1 ",")) ; text in 1st. column
         ; add key to prompt (found in 2nd. column)
         (setq lprmpt (strcat lprmpt "(" (strcol ln 2 ",") ")")) 
         (setq lprmpt (strcat lprmpt "/"))
         (setq prmpt (strcat prmpt lprmpt))
      ) ;end while 
      (close fr)
      ; build sequence command default key at end of prompt
      (setq prmpt (strcat prmpt "<" sqd ">: "))
      ;initialise in case no key is chosen, ie: <return> pressed
      (setq key "") ; which will then use the default (variable: sqd)
      (setq key (strcase (getstring 
         (strcat "\nSequences \"" sqf "\" File(?)/Add(+)/Remove(-)" prmpt))))
      (cond 
        ((= key "?") (filesq)) ;run the file sequences function - end cond1
        ((= key "+") (addsq))  ;run the add sequence function - end cond2
        ((= key "-") (delsq))  ;run the delete sequence function - end cond3
        ((= key "") (progn 
                    (setq key sqd) ;set key to default read from file (line 2)
                    (findkey) ; find the line that this key referrs to
                    (sqopt)   ; choose sequence options
                  )) ; end progn - end cond4
        ; else
        ( T (progn 
            (if (= (iskey key) "False") ; checks prompt line to see if key
              (progn                                                ; exists
                (ALERT "Non-existant key!")
                (exit)
              ) ; end progn
            ) ; end if
            (findkey) ; find the line that this key referrs to
            (sqopt)   ; choose sequence options
          ) ; end progn
        ) ; end cond5
      ) ; end cond
      (echblpres) ; restore original "blipmode" & "echomode" 
      (princ)
  ) ; end progn
  (nofilesq) ; else exit sequence function gracefully and find the file etc. 
  ) ; end if                               
) ; end c:sq
;--------------------------------------------------------------------------;
;  Finds the line of the sequence file containing the key chosen 
;  by the user from the sequence command prompt
;
(defun findkey ()
   (setq fr (open filename "r"))
   (setq ln (read-line fr))  ; read first line  - file name
   (setq ln (read-line fr))  ; read second line - default key 
   (setq theline 0) ;variable used to count until the required key is recorded
   (setq lkey "")   ;intialise variable to hold key found on each line
   (while (/= key lkey) ;end loop when key chosen matches line key
     (setq ln (read-line fr))
     (setq naprmpt (strcol ln 1 ",")) ; text in 1st. column
     (setq lkey (strcol ln 2 ",")) ; text in 2nd. column
     (setq theline (1+ theline))
     (setq sqs (strcol ln 3 ",")) ; find the scale factor in 3rd. column
     (setq sqc (strcol ln 4 ",")) ; find the colour number in 4th. column
     (setq sqtxt (strend ln ",")) ; find the sequence number in last column
  ) ;end while
  (close fr)
) ; end findkey
;--------------------------------------------------------------------------;
;  Sets user option to continue or restart a sequence numbering
;  then directs the program to the correct funtion
;
(defun sqopt ()
   (setq opt (getstring (strcat "\nNext sequence number for \"" 
                naprmpt "(" key ")\" is " sqtxt ". Restart/<Continue>: "))) 
   (setq opt (strcase opt T)) ;to trap upper case "R"
   (if (/= opt "r") (consq) (resq)) ;if not "r" then (consq) else (resq)
) ; end sqopt
;--------------------------------------------------------------------------;
;  Finds relevant next sequence number from file and
;  puts next sequence number & prefix into WBLOCK attribute on drawing
;
(defun consq ()
  (setq ins (getpoint "\nSelect centre insertion point :"))   
  (while (null ins) ;if null insertion, loop until insertion point chosen
    (prompt "\nInsertion point required")
    (setq ins (getpoint "\nSelect centre insertion point :"))     
  ) ; end while
  ; concatenate the appropriate prefix to the sequence number string:
  (setq atr (strcat key sqtxt)) ; sets wblock attribute
  ; Insert the block with the attribute included
  (command "insert" "SQ" ins sqs sqs "0" atr) ; use scale factor from file   
;--------ALTER SCALE OR COLOUR, IF REQUIRED------------------------------   
  (scl) ; run the rescaling function
  ;(clr) ; run alter colour function - DISABLED as not possible with blocks
;--------ADD 1 TO SEQUENCE NUMBER READY TO WRITE BACK TO THE FILE--------    
  ;turn number string into a number and add 1 to the number
  (setq nxtsq (+ (atoi sqtxt) 1))
  (setq sqtxt (itoa nxtsq)) ; turn number back into a string:
  (setq nwsqtxt sqtxt) ; set the variable used in the (updatesq) function
  (updtsq) ;update the sequence file
  (setq *error* olderr) ;reset error function
  (princ)
) ; end consq - END OF MAIN PROGRAMS
;--------------------------------------------------------------------------;
;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^;
                           SUB-PROGRAM FUNCTIONS
;--------------------------------------------------------------------------;
;  Adjusts scale of sequence block, if requested
;
(defun scl ()
  (setq rsz "A")
  (while (and (null (distof rsz 2)) (/= rsz "") (/= (strcase rsz) "N"))
    (setq rsz (getstring 
                "\nResize sequence block by factor [number]/<No change>: "))
    (if (and (null (distof rsz 2)) (/= rsz "") (/= (strcase rsz) "N"))
      (prompt "A number required or just press <return> for <No change>")
    ) ; end if
  ) ;end while, repeat "while" until a real or a blank is entered
  (if (not (null (distof rsz 2))) ; if it is a real
    (progn
      ; multiply new factor by old
      (setq rszr (* (distof rsz 2) (distof sqs 2)))
      (setq sc (distof rsz 2)) ; turn factor string into a real number
      (command "scale" "L" "" ins sc) ; rescale block
      (setq ok (getstring "\nSave new size for future insertions N<Y>: "))
      (if (/= (strcase ok) "N")
        (setq sqs (rtos rszr)) ; set variable used in (updtsq) function
      ) ; end if
    ) ; end progn
  ) ; end if
) ; end scl
;--------------------------------------------------------------------------;
;  Changes colour & saves changes if requested - DISABLED
;  Will not work with blocks but retain for future use
;
;|(defun clr ()
  (ALERT "About to change colour") ; test only
    (setq ok (getstring "\nChange colour: Y<N>: "))
    (if (= (strcase ok) "Y")
      (progn
        (setq nc (acad_colordlg (atoi sqc) nil))
        ;----explode block to change colour
        (command "explode" "L" "")
        (command "chprop" "L" "" "C" nc "")
        (setq ok (getstring "\nSave new colour for future insertions N<Y>: "))
        (if (/= (strcase ok) "N")
          (setq sqc (itoa nc)) ; set variable used in (updtsq) function
        ) ; end if
      ) ; end progn
    ) ; end if
) ; end clr|;
;--------------------------------------------------------------------------;
;  Restarts sequencing at a new number chosen by user
;
(defun resq ()
   (setq sqtxt (itoa (getint "\nRestart sequence at? :")))
   (consq) ; go to the 'insert continue sequence' function
)  ; end resq - END OF MAIN PROGRAMS

;--------------------------------------------------------------------------;
;  Updates the current sequence file to reflect changes
;
;  Creates a variable to hold every line of the "curr.seq" file   
;  while only updating the line to contain the next sequence number text
;  then writes this variable back to the file
;
(defun updtsq ()
   (setq fr (open filename "r"))
   (setq ftxt "")  ;create an empty variable to hold all the text of the file
   ; to allow for the 2 initial non sequence lines not part of the count
   (setq theline (+ 2 theline)) ;add 2 to the value
   (if fr          ;if the file was opened sucessfully
      (progn
         ;read the first line (no "\n" in string) to start building whole file
         (setq ftxt (read-line fr)) 
         (setq line 2)   ; start the line counter at line 2
         (while (setq ln (read-line fr)) ;read the line into variable "ln"
            ; make a variable containing the whole sequence file
            ; by reading each line in turn and adding the strings 
            ; together by concatinating with itself 
            ; including a "\n" before each line
            ; and only incrementing the sequence on appropriate line
            (if (= line theline)               ;if the wanted line is reached
               ; reuse text up to second comma and 
               ; if changed, update scale and/or colour then 
               ; replace sequence number at the end
               (progn
                 ;set text of line up to and including second comma
                 (setq ln (strcat (strcol ln 1 ",") ","
                                                   (strcol ln 2 ",") ","))
                 ;-------SCALE-------
                   (setq ln (strcat ln sqs ",")) ; add scale factor
                 ;-------------------
                 ;-------COLOUR-------
                   (setq ln (strcat ln sqc ",")) ; add colour number
                 ;-------------------
                 ;add new sequence text to the above
                 (setq ftxt (strcat ftxt "\n" ln nwsqtxt)) 
               ) ;end progn 
               ; else it is not "theline"
               (if (= line 2)
                 (setq ftxt (strcat ftxt "\n" key)) ; latest default on line 2
                 ; else
                 (setq ftxt (strcat ftxt "\n" ln))  ;otherwise use existing     
               ) ;end if
            ) ;end if
            (setq line (1+ line)) ; increment line counter before reading         
                                                     ; next line in while loop
         ) ;end while
         (close fr)
      ) ;end progn
   ) ;end if
;  REOPEN THE FILE & WRITE THE FULL CONCATENATED STRING TO IT   
   (updtfls ftxt) ; copy current sequence file back to parent file with changes
) ; end updatesq                                   
;--------------------------------------------------------------------------;    
;
;  REOPEN THE FILE & WRITE THE FULL CONCATENATED STRING TO IT    
;  & copy current sequence file back to parent file with changes
;
;  Usage: (updtfls file_content)
;
(defun updtfls (txt)
  (setq fw (open filename "w"))   
  (write-line txt fw)
  (close fw)
  ;-------
  (setq fil (findfile sqpf))
  (if (null fil)
    (progn
      (ALERT (strcat "Cannot find parent file \"" sqf "\""))
      (setq thefile 
          (getfiled (strcat "Find/Create \"" sqf "\" parent file") 
                                                          sqpf "seq" 8))
      (if (not (null thefile))
        (progn ; 
          (setq sqpf thefile) ; reset the path/name used throughout 
          ; re-write first line, retain the rest
          (setq ftxt sqpf) ;start file text with new path/name of file
          (setq fr (open filename "r"))
          (setq ln (read-line fr)) ; read the first line
          (while (setq ln (read-line fr))
            (setq ftxt (strcat ftxt "\n" ln))
          ) ; end while
          (close fr)
          (setq fw (open filename "w"))
          (write-line ftxt fw)
          (close fw)
        ) ; end progn
      ) ; end if
    ) ; end progn
  ) ; end if
  ;-------
  (fcopy filename sqpf) ; copy current sequence file back to parent file
) ; end updatesq                                   ; with the latest changes
;--------------------------------------------------------------------------;

;  Checks current "blipmode" & "echomode"
;  and stores these values in global variables to return to later
;  and turns "blipmode" & "echomode" off
;
(defun echblpoff ()
   (setq sblip (getvar "blipmode"))
   (setq scmde (getvar "cmdecho"))
   (setvar "blipmode" 0)
   (setvar "cmdecho" 0)
) ; end echblpoff
;--------------------------------------------------------------------------;
;  Restores "blipmode" & "echomode" to original values
;
(defun echblpres ()
   (setvar "blipmode" sblip)
   (setvar "cmdecho" scmde)
) end echblpres
;--------------------------------------------------------------------------;
;  Adds new sequence and key to sequence file
;
(defun addsq ()
   (addna) ; asks for a Name, checks to see if Name already exists,
   ;                 if if does, it goes back to the beginning of (addna)
   ------------------
   (addkey) ; asks for a Key, checks to see if Key already exists,
   ;                 if it does, it goes back to the beginning of (addkey)
   ------------------
   (setq prmptstr (strcat 
                        "\nStart Sequence for \"" sqna "(" sqkey ")\" at: "))
   (setq sqstarttxt (itoa (getint prmptstr)))
   (setq ok (getstring (strcat "\nAre you sure you want to add \""
                       sqna "(" sqkey ")\" to the \"" sqf "\" file? N<Y>:")))
   ; append file with new sequence details if affirmed
   (if (/= (strcase ok) "N")
     (progn   
       (setq fa (open filename "a")) ; in "append" mode
       ; ",1.0000,7," referrs to: 1.0000: scale factor, 7: the colour white
       (setq nwline (strcat sqna "," sqkey ",1.0000,7,"  sqstarttxt))
       (write-line nwline fa)
       (close fa)
       (fcopy filename sqpf) ; copy current sequence file back to parent file
                                                          ;with latest changes
       (setq nwsqtxt sqstarttxt)
       (setq key sqkey)
       (updtsq) ; updates the default line only
     ) ;end progn                                         
   ) ;end if
) ; end addsq
;-------------------------------------------------------------------------;
;  Check to see if new sequence name already exists in file
;  by looking along the earlier built prompt string "prmpt"
;
(defun addna ()
  (setq sqna (getstring "\nNew sequence Name: "))
  ; prevent a zero length name being attempted
  (while (< (strlen sqna) 1)
    (setq sqna (getstring "\nNew sequence Name: "))
  ) ; end while
  ; ADD LINES to make sure it is not a number
  (setq sqna (strcase sqna T)) ;make sure all chars are lower case
  ; then make the first char UPPER case
  (setq char1 (strcase (substr sqna 1 1)))
  ; rebuild the string
  (setq sqna (strcat char1 (substr sqna 2)))
  (setq chars "")
  ; the format "/" "Name" "(" always occurrs, so this concatenation
  ; ensures that the string cannot be found as part of another name
  ; eg: "lot" is within "plot", but "/Lot(" is NOT within "/Plot("
  (setq nastr (strcat  "/" sqna "("))
  (setq pos 1)
  (setq len (strlen nastr)) ; get the lenght of the resulting string
  (while  (and (/= chars nastr) (<= pos (strlen prmpt)))
    (setq chars (substr prmpt pos len))
    (setq pos (1+ pos))
  ) ; end while
  (if (= chars nastr) ; if the name already exists
    (progn
      (alert (strcat "The sequence name \"" sqna "\"\nalready exists"
                           "\nPlease choose another"))
      (addna) ; go back and start again
    ) ; end progn
  ) ; end if
) ; end addna
;-------------------------------------------------------------------------;
;  Check to see if new sequence key already exists in file
;  by looking along the earlier built prompt string "prmpt"
;
(defun addkey ()
  (setq sqkey (getstring 
       (strcat "\nSequence Key for \"" sqna 
                            "\" (1 or 2 characters, end alphabetically): ")))
  ; prevent less than 1 or more than 2 characters being used
  ; and prevent a number being used at the end of the string
  ; or this may be confused with part of the sequence number itself
  ; excludes the numbers which are ascii codes 55 to 64
  (while (or (< (strlen sqkey) 1) (> (strlen sqkey) 2)
                 (> 64 (ascii (substr sqkey (strlen sqkey) 1))))
    (prompt (strcat 
    "Use 1 or 2 characters, end alphabetically, for key to \"" sqna "\""))
    (setq sqkey (getstring (strcat "\nSequence Key for \"" sqna 
                          "\" (1 or 2 characters, end alphabetically): ")))
  ) ;end while
  (setq sqkey (strcase sqkey)) ; make the key all UPPER case
  (if (= (iskey sqkey) "True") ; function to see if the key already exists
    (progn
      (alert (strcat "The key \"" 
        sqkey "\" chosen for\nsequence name \"" sqna "\"\nis already in use"
                           "\nPlease choose another"))
      (addkey) ; go back and start again
    ) ; end progn
  ) ; end if
) ; end addkey
;-------------------------------------------------------------------------;
;  Deletes a sequence from sequence file
;
(defun delsq ()
  (setq key (strcase (getstring (strcat "\nDelete sequence " prmpt))))
  (if (= (iskey key) "False") ; function to see if key exists
    (progn
      (ALERT "Non-existant key!")
      (exit)
    ) ; end progn
  ) ; end if
  ;-----------
  ; put sequence name in deletion prompt, "pos" assigned in (iskey key)
  ; called above & used as starting point to search for previous "/"
  (setq naprmpt (strend (substr prmpt 1 (- pos 2)) "/"))
  ;-----------
  (setq ok (getstring (strcat "\nAre you sure you want to delete \"" 
                   naprmpt "(" key ")\" from the \"" sqf "\" file? N<Y>: ")))
  ; append file with new sequence details if affirmed
  (if (/= (strcase ok) "N")
    (progn
      (findkey)
;------CREATE A VARIABE TO HOLD EVERY LINE OF THE "SEQ.TXT" FILE------   
;------WHILE ONLY DELETING THE REQUIRED LINE
      (setq fr (open filename "r"))
      (setq ftxt "")  ; create an empty variable to hold all the text of the file
      (setq line 1)   ; start the line counter at line 1 
      ; to allow for the 2 initial lines not part of the count
      (setq theline (+ 2 theline)) ;add 2 to the value to allow for 1st. 2 lines
      (if fr          ; if the file was opened sucessfully
        (progn
          (setq ln (read-line fr))
          (if (/= line theline) ; if not the line to be omitted
            (setq ftxt (strcat ftxt ln))      
          ) ; end if
          (setq line 2)
          (while (setq ln (read-line fr)) ;read line into variable "ln"
            ; make a variable containing the whole sequence file
            ; by reading each line in turn and adding the strings 
            ; together by concatinating with itself 
            ; and only omitting the sequence of appropriate line
            (if (/= line theline)       ;if not the line to be omitted
              (setq ftxt (strcat ftxt "\n" ln))      
            ) ;end if
            (setq line (1+ line))
          ) ;end while
          (close fr)
        ) ;end progn
      ) ;end if
;  REOPEN THE FILE & WRITE THE FULL CONCATENATED STRING TO IT   
   (updtfls ftxt) ; copy current sequence file back to parent file
      ) ; end progn                                    ; with latest changes
  ) ; end if                                          
) ; end delsq                                         
;--------------------------------------------------------------------------;
;  Alerts user that unable to open "curr.seq" (for whatever reason),
;  exits sequence function and attempts to find
;  the current sequence file, then returns sequence function
;
(defun nofilesq ()
   (alert (strcat "Cannot find/open current sequence file: " filename))
   ; performs library search for file & opens file dialog
   (setq filename (getfiled "Find/Create current sequence file" 
                                                     "curr" "seq" 8))
   (if (null filename)
     (progn
       (setq ok (strcase(getstring "\nCreate new sequence file?/N<Y>: ")))
       (if (or (= ok "") (= ok "Y")) 
         (nwcurrsq) ; new current sequence file       
         (exit) ; else quit the sequence function
       ) ;end if
     ) ;end progn
     (progn ; otherwise......
       (fcopy filename "c:\curr.seq") ; (could change path)
       (alert (strcat "The path/name of the current sequence file\n"
            "         is: \"" filename "\"."
            "\nThe file \"" (strend filename "\\") "\" will be copied to the"
            "\nroot directory for easy access in future."))
     ) ;end progn
   ) ;end if
   (setq filename "c:\curr.seq") ; reset the variable to standard
) ; end nofilesq
;--------------------------------------------------------------------------;
;  Creates new current sequence file based on new "*.seq" file
;
(defun nwcurrsq ()
  (setq filename "c:\curr.seq")
  (setq title "New Sequence File")
  (setq flag 9)
  (filesq1)
  (setq fw (open sqpf "w"))
  ; put name of file on first line & add "\n+" (new line with default) 
  (write-line (strcat sqpf "\n+") fw)  
  (close fw)
  (fcopy sqpf filename)
  (addsq)
) ; end nwcurrsq
;--------------------------------------------------------------------------;
;  Changes or adds new sequence file (with first entry) to any directory
;  location and updates the current sequence file to reflect the change
;
(defun filesq ()
  (setq fprmpt  
        (strcat "\nCurrent Sequence file is \"" sqf "\" Change(C)/New(N) :"))
  (setq opt (strcase (getstring fprmpt)))
  (cond
    ((= opt "C")
      (progn
        (setq title "Change Sequence File")
        (setq flag 8)
        (filesq1)
        (fcopy sqpf filename)
      )) ; end cond1
    ((= opt "N")
      (progn
        (setq title "New Sequence File")
        (setq flag 9)
        (filesq1)
        (setq fw (open sqpf "w"))
        ; put name of file on first line & add "\n+" (new line with default) 
        (write-line (strcat sqpf "\n+") fw)  
        (close fw)
        (fcopy sqpf filename)
        (addsq)
      )) ; end progn, end cond2
  ) ; end cond
) ; end filesq
;--------------------------------------------------------------------------
;  Copies current sequence file to the file named on 1st. line of "curr.seq"
;
(defun filesq1 ()
  (fcopy filename sqpf)
  (setq sqpf (getfiled title "" "seq" flag))
) ; end filesq1
;-------------------------------------------------------------------------
;   Copies first file into second. This program takes two ASCII files   
;   as arguments, If the first file does not exist, an error message 
;   is printed, however, if the second file does not exist, it is created.
;
;   Note that if the second file exists, its data will be overwritten.
;  
;   Usage: (fcopy "infile.ext" "outfile.ext")

(defun fcopy (in out / fr fw ln) 
  (cond ((null (setq fr (open in "r"))) ; try to open in for reading
     (princ "Can't open ")            ; if nil print error message
     (princ in) 
     (princ " for reading.")
    ) 
    (if ifp 
      (progn
        (setq fw (open out "w"))     ; else open out for writing
        (while (setq ln (read-line fr)) ; read each line from in
          (write-line ln fw)          ; and write it to out
        ) ;end while
        (close fw)                   ; close both files
        (close fr)
      ) ;end progn
    ) ;end if
  ) ;end cond
  (princ)
) ; end fcopy

;--------------------------------------------------------------------------;
;   Finds "string" after last ocurrence of "str"
;   by working back from last character of "string"
;
;   Usage: (strend "string" "after character")
;
(defun strend (string str)
   (setq char "")
   (setq pos (strlen string)) ; set "pos" to length of "string"
   (while (/= char str) 
   (setq char (substr string pos 1)) ; check each "char" until equals "str"
     (setq pos (1- pos)) ; count backwards by 1
   ) ; end while
   (substr string (+ 2 pos)) ; return resulting substring after "str"
) ; end strend
;--------------------------------------------------------------------------;
;  Finds substring in string, in column number, separated by character
;
;  Sample string :"Doors,D,1.500,5,1001"
;
;  Usage: (strcol string column separator)
;
(defun strcol (str col schar)
  (setq str (strcat str schar)) ; add extra separator at end of string
  (setq colno 0) ; initiate the column counter
  (setq posb 1) ; initiate the 2nd. position indicator
  (while (and (/= colno col) (<= posb (strlen str))) ; while the correct column has not been reached
    ; initiate 1st. position indicator which is 1 character
    (setq posa (+ 1 posb)) ; after the previous position indicator
    ; while the correct character has not been found and
    ; the end of the line has not been reached
    (setq char "")
    (while (and (/= char schar) (<= posb (strlen str))) 
      (setq char (substr str posb 1)) ; read one character
      (setq posb (1+ posb)) ; increment the 2nd. position indicator
    ) ; end while
    (if (= char schar)
      (setq colno (1+ colno)) ; increment the column counter
    )
  ) ; end while
  ;(ALERT (strcat "At column :" (itoa colno) "\nposa = " (itoa posa) 
               ; "\nposb = " (itoa posb))
  (substr str (- posa 1) (- posb posa)) ; returns text between separators
) ; end strcol
;--------------------------------------------------------------------------;
;  Check to see if user has typed characters that exist as a key
;  in the prompt line
;
(defun iskey (thekey)
  (setq chrs "")
  ; the format "(" "thekey" ")" always occurrs, so this concatenation
  ; ensures that the string cannot be found as part of another key
  ; eg: "B" is within "BL", but "(B)" is NOT within "(BL)"
  (setq kystr (strcat  "(" thekey ")"))
  (setq pos 1)
  (setq ln (strlen kystr)) ; get the lenght of the resulting string
  (while  (and (/= chrs kystr) (<= pos (strlen prmpt)))
    (setq chrs (substr prmpt pos ln))
    (setq pos (1+ pos))
  ) ; end while
  (if (= chrs kystr) ; if the key already exists
    (setq tr "True") ;   return True
    (setq tr "False") ; else return False
  ) ; end if
) ; end iskey
;--------------------------------------------------------------------------;
;  Sorts a list of strings passed to it and returns the sorted list 
;  DISABLED
;
;|(defun c:sort (strlst)
  (acad_strlsort strlst)
  ; (ALERT (strcat "The sorted string is: " stxt)) ; for testing
)|;
;--------------------------------------------------------------------------;
